@@ -22,21 +22,103 @@ class KotlinBenchmarkManager {
                 private const val TAG = "KotlinBenchmarkManager"
 
 
-        private val SCORING_FACTORS =
-        mapOf(
-                // Target 20 / Performance (Mops/s)
-                BenchmarkName.PRIME_GENERATION to 1.7985e-6*2,        // 20 / 2.90e6 ops/s        
-                BenchmarkName.FIBONACCI_ITERATIVE to 4.365e-7,     // 20 / 22.91 Mops/s
-                BenchmarkName.MATRIX_MULTIPLICATION to 1.56465e-8/4,  // 20 / 639.13 Mops/s
-                BenchmarkName.HASH_COMPUTING to 2.778e-5/2,          // 20 / 0.36 Mops/s
-                BenchmarkName.STRING_SORTING to 1.602e-7/2,          // 20 / 62.42 Mops/s
-                BenchmarkName.RAY_TRACING to 4.902e-6,             // 20 / 2.04 Mops/s
-                BenchmarkName.COMPRESSION to 1.5243e-8,            // 20 / 656.04 Mops/s
-                BenchmarkName.MONTE_CARLO to 0.6125e-6,             // 20 / 16.32 Mops/s
-                BenchmarkName.JSON_PARSING to 1.56e-6*4,            // 20 / 6.41 Mops/s
-                BenchmarkName.N_QUEENS to 2.011e-7/2                 // 20 / 66.18e6 ops/s
+        // Reference device: Snapdragon 8 Gen 3 (OnePlus Pad 2)
+        // These are the baseline ops/s values used for geometric mean calculation
+        // Note: Values are in ops/s, not Mops/s, to match benchmark result format
+        val REFERENCE_MOPS = mapOf(
+                BenchmarkName.PRIME_GENERATION to 757_430_000.0,           // 571.43 Mops/s (Pollard's Rho)
+                BenchmarkName.FIBONACCI_ITERATIVE to 4_560_000.0,          // 4.56 Mops/s
+                BenchmarkName.MATRIX_MULTIPLICATION to 3_876_440_000.0,    // 3876.44 Mops/s
+                BenchmarkName.HASH_COMPUTING to 138_370_000.0,             // 138.37 Mops/s
+                BenchmarkName.STRING_SORTING to 125_200_000.0,             // 125.20 Mops/s
+                BenchmarkName.RAY_TRACING to 8_820_000.0,                 // 8.82 Mops/s (Perlin Noise)
+                BenchmarkName.COMPRESSION to 758_880_000.0,                // 758.88 Mops/s
+                BenchmarkName.MONTE_CARLO to 280_460_000.0,               // 229.46 Mops/s (Mandelbrot Set)
+                BenchmarkName.JSON_PARSING to 188_503_800_000.0,            // 91503.80 Mops/s
+                BenchmarkName.N_QUEENS to 166_790_000.0                    // 166.79 Mops/s
         )
 
+        val SCORING_FACTORS =
+        mapOf(
+                // Target 20 / Performance (Mops/s)
+                BenchmarkName.PRIME_GENERATION to 1.7985e-6/132.6,        // 20 / 2.90e6 ops/s        
+                BenchmarkName.FIBONACCI_ITERATIVE to 4.365e-7*5,     // 20 / 22.91 Mops/s
+                BenchmarkName.MATRIX_MULTIPLICATION to 1.56465e-8/7.2,  // 20 / 639.13 Mops/s
+                BenchmarkName.HASH_COMPUTING to 2.778e-5/384,          // 20 / 0.36 Mops/s
+                BenchmarkName.STRING_SORTING to 1.602e-7/2,          // 20 / 62.42 Mops/s
+                BenchmarkName.RAY_TRACING to 4.902e-6/4.2,             // 20 / 2.04 Mops/s
+                BenchmarkName.COMPRESSION to 1.5243e-8*0.92,            // 20 / 656.04 Mops/s
+                BenchmarkName.MONTE_CARLO to 0.6125e-6/20,             // 20 / 16.32 Mops/s
+                BenchmarkName.JSON_PARSING to 1.56e-6/28500,            // 20 / 6.41 Mops/s
+                BenchmarkName.N_QUEENS to 2.011e-7/3.2                 // 20 / 66.18e6 ops/s
+        )
+
+        }
+
+        /**
+         * Run test workload before actual benchmarks
+         * - Uses minimal "test" workload parameters
+         * - No delays between benchmarks
+         * - Results are NOT recorded
+         * - Purpose: Warm up device and stabilize performance
+         */
+        private suspend fun runTestWorkload() {
+                Log.d(TAG, "=== STARTING TEST WORKLOAD (Warm-up) ===")
+                val testParams = getWorkloadParams("test")
+                val isTestRun = true
+
+                // Emit test workload start event
+                _benchmarkEvents.emit(
+                        BenchmarkEvent(
+                                testName = "Test Workload",
+                                mode = "TEST",
+                                state = "STARTED",
+                                timeMs = 0,
+                                score = 0.0
+                        )
+                )
+
+                // Run all benchmarks with test parameters (no recording, no delays)
+                try {
+                        // Single-core test benchmarks
+                        SingleCoreBenchmarks.primeGeneration(testParams, isTestRun)
+                        SingleCoreBenchmarks.fibonacciRecursive(testParams, isTestRun)
+                        SingleCoreBenchmarks.matrixMultiplication(testParams, isTestRun)
+                        SingleCoreBenchmarks.hashComputing(testParams, isTestRun)
+                        SingleCoreBenchmarks.stringSorting(testParams, isTestRun)
+                        SingleCoreBenchmarks.rayTracing(testParams, isTestRun)
+                        SingleCoreBenchmarks.compression(testParams, isTestRun)
+                        SingleCoreBenchmarks.monteCarloPi(testParams, isTestRun)
+                        SingleCoreBenchmarks.jsonParsing(testParams, isTestRun)
+                        SingleCoreBenchmarks.nqueens(testParams, isTestRun)
+
+                        // Multi-core test benchmarks
+                        MultiCoreBenchmarks.primeGeneration(testParams, isTestRun)
+                        MultiCoreBenchmarks.fibonacciRecursive(testParams, isTestRun)
+                        MultiCoreBenchmarks.matrixMultiplication(testParams, isTestRun)
+                        MultiCoreBenchmarks.hashComputing(testParams, isTestRun)
+                        MultiCoreBenchmarks.stringSorting(testParams, isTestRun)
+                        MultiCoreBenchmarks.rayTracing(testParams, isTestRun)
+                        MultiCoreBenchmarks.compression(testParams, isTestRun)
+                        MultiCoreBenchmarks.monteCarloPi(testParams, isTestRun)
+                        MultiCoreBenchmarks.jsonParsing(testParams, isTestRun)
+                        MultiCoreBenchmarks.nqueens(testParams, isTestRun)
+
+                        Log.d(TAG, "=== TEST WORKLOAD COMPLETE ===")
+                } catch (e: Exception) {
+                        Log.w(TAG, "Test workload encountered error (non-critical): ${e.message}")
+                }
+
+                // Emit test workload complete event
+                _benchmarkEvents.emit(
+                        BenchmarkEvent(
+                                testName = "Test Workload",
+                                mode = "TEST",
+                                state = "COMPLETED",
+                                timeMs = 0,
+                                score = 0.0
+                        )
+                )
         }
 
         suspend fun runAllBenchmarks(deviceTier: String = "Flagship") {
@@ -44,6 +126,10 @@ class KotlinBenchmarkManager {
                         TAG,
                         "SINGLE_SOURCE_OF_TRUTH: Starting benchmark execution with device tier: $deviceTier"
                 )
+
+                // Run test workload first (warm-up)
+                runTestWorkload()
+
                 val params = getWorkloadParams(deviceTier)
 
                 // Log CPU topology
@@ -372,25 +458,65 @@ class KotlinBenchmarkManager {
                 }
         }
 
+        /**
+         * Calculate geometric mean score for benchmark results.
+         * 
+         * Formula: GeometricMean = (∏ ratios)^(1/n)
+         * where ratio = SUT_Mops / Reference_Mops
+         * 
+         * This prevents one fast benchmark from dominating the score and provides
+         * fair comparison across different device architectures.
+         * 
+         * @param results List of benchmark results
+         * @return Geometric mean score scaled to 100-point baseline (SD 8 Gen 3 = 100)
+         */
+        private fun calculateGeometricMean(results: List<BenchmarkResult>): Double {
+                if (results.isEmpty()) return 0.0
+                
+                // Calculate performance ratios for each benchmark
+                val ratios = results.mapNotNull { result ->
+                        val benchmarkName = BenchmarkName.fromString(result.name)
+                        val refMops = benchmarkName?.let { REFERENCE_MOPS[it] }
+                        
+                        if (refMops != null && refMops > 0.0) {
+                                // Ratio = SUT performance / Reference performance
+                                result.opsPerSecond / refMops
+                        } else {
+                                Log.w(TAG, "No reference value for ${result.name}, skipping in geometric mean")
+                                null
+                        }
+                }
+                
+                if (ratios.isEmpty()) {
+                        Log.e(TAG, "No valid ratios calculated for geometric mean")
+                        return 0.0
+                }
+                
+                // Calculate geometric mean: (product)^(1/n)
+                var product = 1.0
+                for (ratio in ratios) {
+                        product *= ratio
+                }
+                
+                val geometricMean = Math.pow(product, 1.0 / ratios.size)
+                
+                // Scale to 100-point baseline (SD 8 Gen 3 = 100)
+                val score = geometricMean * 100.0
+                
+                Log.d(TAG, "Geometric mean calculation: ${ratios.size} benchmarks, GM=${geometricMean}, Score=${score}")
+                
+                return score
+        }
+
         private fun calculateSummary(
                 singleResults: List<BenchmarkResult>,
                 multiResults: List<BenchmarkResult>
         ): String {
-                // Calculate single-core score using weighted scoring
-                var calculatedSingleCoreScore = 0.0
-                for (result in singleResults) {
-                        val benchmarkName = BenchmarkName.fromString(result.name)
-                        val factor = benchmarkName?.let { SCORING_FACTORS[it] } ?: 0.0
-                        calculatedSingleCoreScore += result.opsPerSecond * factor
-                }
+                // Calculate single-core score using geometric mean
+                val calculatedSingleCoreScore = calculateGeometricMean(singleResults)
 
-                // Calculate multi-core score using weighted scoring
-                var calculatedMultiCoreScore = 0.0
-                for (result in multiResults) {
-                        val benchmarkName = BenchmarkName.fromString(result.name)
-                        val factor = benchmarkName?.let { SCORING_FACTORS[it] } ?: 0.0
-                        calculatedMultiCoreScore += result.opsPerSecond * factor
-                }
+                // Calculate multi-core score using geometric mean
+                val calculatedMultiCoreScore = calculateGeometricMean(multiResults)
 
                 // Calculate final weighted score (35% single, 65% multi)
                 val calculatedFinalScore =
@@ -516,76 +642,74 @@ class KotlinBenchmarkManager {
 
         private fun getWorkloadParams(deviceTier: String): WorkloadParams {
                 return when (deviceTier.lowercase()) {
+                        "test" ->
+                                WorkloadParams(
+                                        primeRange = 12_250_000,  // 0.1x slow
+                                        fibonacciNRange = Pair(92, 92),
+                                        fibonacciIterations = 520_833,  // 0.1x slow
+                                        matrixSize = 128,
+                                        matrixIterations = 37,  // 0.1x slow (rounded from 37.5)
+                                        hashDataSizeMb = 8,
+                                        hashIterations = 6_568_750,  // 0.1x slow
+                                        stringSortIterations = 62,  // 0.1x slow (rounded from 62.5)
+                                        rayTracingIterations = 10,  // 0.1x slow
+                                        rayTracingResolution = Pair(256, 256),
+                                        rayTracingDepth = 5,
+                                        compressionDataSizeMb = 2,
+                                        compressionIterations = 25,  // 0.1x slow
+                                        monteCarloSamples = 625_000L,  // 0.1x slow
+                                        jsonDataSizeMb = 1,
+                                        jsonParsingIterations = 50,  // 0.1x slow (rounded from 31.25)
+                                        nqueensSize = 12  // N-Queens: keep same as slow
+                                )
                         "slow" ->
                                 WorkloadParams(
-                                        primeRange = 10_000_000,
-
-                                        // ~1.0-2.0s
+                                        primeRange = 122_500_000,  // 0.25x mid
                                         fibonacciNRange = Pair(92, 92),
-                                        fibonacciIterations = 75_000_000,
-                                        matrixSize = 128, // CACHE-RESIDENT: Fixed small size
-                                        matrixIterations =
-                                                800, // CACHE-RESIDENT: Low iterations for slow
-                                        // devices
+                                        fibonacciIterations = 5_208_333,  // 0.25x mid
+                                        matrixSize = 128,
+                                        matrixIterations = 375,  // 0.25x mid
                                         hashDataSizeMb = 8,
-                                        hashIterations =
-                                                800_000, // FIXED WORK PER CORE: Target ~1.5-2.0
-                                        // seconds
-                                        stringSortIterations =
-                                                800, // CACHE-RESIDENT: Explicit control - target
-                                        rayTracingIterations = 50,
-                                        rayTracingResolution = Pair(100, 100),
-                                        rayTracingDepth = 2,
-                                        compressionDataSizeMb = 1,
-                                        compressionIterations =
-                                                600, // INCREASED: Target ~15s execution (was 20)
-                                        monteCarloSamples =
-                                                10_000_000, // FIXED WORK PER CORE: Target ~1.5s
+                                        hashIterations = 65_687_500,  // 0.25x mid
+                                        stringSortIterations = 625,  // 0.25x mid
+                                        rayTracingIterations = 100,  // 0.25x mid
+                                        rayTracingResolution = Pair(256, 256),
+                                        rayTracingDepth = 5,
+                                        compressionDataSizeMb = 2,
+                                        compressionIterations = 250,  // 0.25x mid
+                                        monteCarloSamples = 6_250_000L,  // 0.25x mid
                                         jsonDataSizeMb = 1,
-                                        jsonParsingIterations =
-                                                500, // CACHE-RESIDENT: Low iterations for slow
-                                        // devices (~1-2s)
-                                        nqueensSize = 12 // INCREASED: 92 solutions, ~1.5s (was 8)
+                                        jsonParsingIterations = 312,  // 0.25x mid (rounded from 312.5)
+                                        nqueensSize = 12  // N-Queens: -1 from mid
                                 )
                         "mid" ->
                                 WorkloadParams(
-                                        primeRange = 25_000_000,
-                                        // ~1.0-2.0s
-                                        fibonacciNRange = Pair(96, 96),
-                                        fibonacciIterations = 75_000_000,
-                                        matrixSize = 128, // CACHE-RESIDENT: Fixed small size
-                                        matrixIterations =
-                                                1500, // CACHE-RESIDENT: Medium iterations for mid
-                                        // devices
-                                        hashDataSizeMb = 2,
-                                        hashIterations =
-                                                1_500_000, // FIXED WORK PER CORE: Target ~1.5-2.0
-                                        // seconds
-                                        stringSortIterations =
-                                                2_500, // CACHE-RESIDENT: Explicit control - target
-                                        rayTracingIterations =
-                                                200, // FIXED: Medium iterations for mid devices
-                                        rayTracingResolution = Pair(100, 100),
-                                        rayTracingDepth = 3,
-                                        compressionDataSizeMb = 1,
-                                        compressionIterations =
-                                                1_000, // INCREASED: Target ~15s execution (was 50)
-                                        monteCarloSamples =
-                                                20_000_000, // FIXED WORK PER CORE: Target ~1.5s
+                                        primeRange = 490_000_000,  // 0.5x flagship
+                                        fibonacciNRange = Pair(92, 92),
+                                        fibonacciIterations = 20_833_333,  // 0.5x flagship (rounded from 20833333.5)
+                                        matrixSize = 128,
+                                        matrixIterations = 1_500,  // 0.5x flagship
+                                        hashDataSizeMb = 8,
+                                        hashIterations = 262_750_000,  // 0.5x flagship
+                                        stringSortIterations = 2_500,  // 0.5x flagship
+                                        rayTracingIterations = 400,  // 0.5x flagship
+                                        rayTracingResolution = Pair(256, 256),
+                                        rayTracingDepth = 5,
+                                        compressionDataSizeMb = 2,
+                                        compressionIterations = 1_000,  // 0.5x flagship
+                                        monteCarloSamples = 25_000_000L,  // 0.5x flagship
                                         jsonDataSizeMb = 1,
-                                        jsonParsingIterations =
-                                                4_000, // CACHE-RESIDENT: Medium iterations for mid
-                                        // devices (~1-2s)
-                                        nqueensSize = 13 // INCREASED: 341 solutions, ~5s (was 9)
+                                        jsonParsingIterations = 1_250,  // 0.5x flagship
+                                        nqueensSize = 15  // N-Queens: -1 from flagship
                                 )
                         "flagship" ->
                                 WorkloadParams(
                                         // CACHE-RESIDENT STRATEGY: Small matrices with high
                                         // iterations
 
-                                        primeRange = 50_000_000,
+                                        primeRange = 980_000_000,  // Miller-Rabin: ~40-50s
                                         fibonacciNRange = Pair(92, 92), // Use fixed max safe value
-                                        fibonacciIterations = 125_000_000,
+                                        fibonacciIterations = 41_666_667,  // Reduced 3x for polynomial
                                         matrixSize =
                                                 128, // CACHE-RESIDENT: Fixed small size for cache
                                         // efficiency
@@ -594,7 +718,7 @@ class KotlinBenchmarkManager {
                                         // flagship devices
                                         hashDataSizeMb = 8,
                                         hashIterations =
-                                                2_500_000, // FIXED WORK PER CORE: Target ~1.5-2.0
+                                                525_500_000, // FIXED WORK PER CORE: Target ~1.5-2.0
                                         // seconds
 
                                         stringSortIterations =
@@ -617,15 +741,11 @@ class KotlinBenchmarkManager {
                                                 2_000, // INCREASED: Target ~15s execution (was
                                         // 100)
                                         monteCarloSamples =
-                                                50_000_000, // FIXED WORK PER CORE: Target ~1.5s
-                                        // (was
-                                        // 15M)
+                                                50_000_000L, // Reduced 100x for Mandelbrot Set (was 5B)
                                         jsonDataSizeMb = 1,
-                                        jsonParsingIterations =
-                                                800, // CACHE-RESIDENT: High iterations for flagship
-                                        // devices (~1-2s)
+                                        jsonParsingIterations = 2500,  // Reduced 10x for CPU-bound parsing
                                         nqueensSize =
-                                                15 // INCREASED: 14,200 solutions, ~20s (was 10)
+                                                16 // INCREASED: 14,200 solutions, ~20s (was 10)
                                 )
                         else -> WorkloadParams() // Default values
                 }
