@@ -61,13 +61,9 @@ fun PowerConsumptionGraph(
     // This creates an inverted Y-axis where negative values appear above the X-axis
     val zeroPosition = if (fullRange != 0f) (0f - minPower) / fullRange else 0.5f
     
-    Card(
+    GlassCard(
         modifier = modifier.fillMaxWidth(),
-        colors = CardDefaults.cardColors(
-            containerColor = MaterialTheme.colorScheme.surface,
-            contentColor = MaterialTheme.colorScheme.onSurface
-        ),
-        elevation = CardDefaults.cardElevation(defaultElevation = 2.dp)
+        shape = androidx.compose.foundation.shape.RoundedCornerShape(24.dp)
     ) {
         Column(
             modifier = Modifier
