@@ -34,6 +34,7 @@ data class TestState(
         val name: String,
         val status: TestStatus,
         val timeText: String = "", // ADDED: Will contain timing like "342ms"
+        val durationMs: Long = 0L, // ADDED: Raw duration for calculation
         val result: com.ivarna.finalbenchmark2.cpuBenchmark.BenchmarkResult? = null
 )
 
@@ -57,7 +58,8 @@ data class BenchmarkUiState(
         val testStates: List<TestState> =
                 emptyList(), // CHANGED: from allTestStates to testStates for clarity
         val workloadPreset: String =
-                "Auto" // ADDED: Track the current workload preset for UI display
+                "Auto", // ADDED: Track the current workload preset for UI display
+        val estimatedTimeRemaining: String = "--:--"
 )
 
 // Old data class kept for compatibility
@@ -90,16 +92,23 @@ class BenchmarkViewModel(
         private val historyRepository:
                 com.ivarna.finalbenchmark2.data.repository.HistoryRepository? =
                 null,
-        private val application: Application,
-        private val onBenchmarkCompleteCallback: ((String) -> Unit)? = null
+        private val application: Application
 ) : ViewModel() {
         private val _benchmarkState = MutableStateFlow<BenchmarkState>(BenchmarkState.Idle)
         val benchmarkState: StateFlow<BenchmarkState> = _benchmarkState
 
+        // SharedFlow for navigation events (One-time events)
+        private val _completionEvent = kotlinx.coroutines.flow.MutableSharedFlow<String>()
+        val completionEvent: kotlinx.coroutines.flow.SharedFlow<String> = _completionEvent
+
         // New state flow for granular benchmark UI state
         private val _uiState = MutableStateFlow(BenchmarkUiState())
         val uiState: StateFlow<BenchmarkUiState> = _uiState
 
+        // New state flow for warm-up status
+        private val _isWarmingUp = MutableStateFlow(false)
+        val isWarmingUp: StateFlow<Boolean> = _isWarmingUp
+
         private val benchmarkManager =
                 com.ivarna.finalbenchmark2.cpuBenchmark.KotlinBenchmarkManager()
         private val cpuUtils = CpuUtilizationUtils(application)
@@ -134,11 +143,7 @@ class BenchmarkViewModel(
                                         // monitoring
                                         // LOG 1: Monitor wakes up
                                         val currentSizeBefore = _uiState.value.completedTests.size
-                                        Log.d(
-                                                "BENCH_DEBUG",
-                                                "[Monitor] Waking up. Current List Size: $currentSizeBefore"
-                                        )
-
+                                        
                                         // Calculate memory usage percentage
                                         val memoryLoad =
                                                 try {
@@ -174,16 +179,44 @@ class BenchmarkViewModel(
 
                                         // CRITICAL MOMENT: Updating State
                                         _uiState.update { currentState ->
-                                                // LOG 2: Inside the atomic update block for Monitor
-                                                if (currentState.completedTests.size !=
-                                                                currentSizeBefore
-                                                ) {
-                                                        Log.d(
-                                                                "BENCH_DEBUG",
-                                                                "[Monitor] !!! RACE CONDITION DETECTED !!! I saw size $currentSizeBefore, but inside update it is ${currentState.completedTests.size}"
-                                                        )
+                                                
+                                                // Calculate Estimated Time Remaining
+                                                var timeRemainingStr = currentState.estimatedTimeRemaining
+                                                if (currentState.isRunning) {
+                                                    val completed = currentState.testStates.filter { it.status == TestStatus.COMPLETED }
+                                                    val remaining = currentState.testStates.filter { it.status != TestStatus.COMPLETED }
+                                                    
+                                                    if (remaining.isNotEmpty()) {
+                                                        val avgTimeMs = if (completed.isNotEmpty()) {
+                                                            completed.map { it.durationMs }.average()
+                                                        } else {
+                                                            // Better initial guess based on preset
+                                                            when (currentState.workloadPreset.lowercase()) {
+                                                                "flagship" -> 9000.0 // ~3 min total (High end 1.5m, Low end 6m -> Geom Mean ~3m)
+                                                                "mid" -> 4500.0
+                                                                "slow" -> 1500.0
+                                                                else -> 2000.0
+                                                            } 
+                                                        }
+                                                        
+                                                        val totalRemainingMs = (avgTimeMs * remaining.size).toLong()
+                                                        
+                                                        // Smoother display: round up to next second
+                                                        val totalSeconds = (totalRemainingMs + 999) / 1000
+                                                        val seconds = totalSeconds % 60
+                                                        val minutes = totalSeconds / 60
+                                                        timeRemainingStr = String.format("%02d:%02d", minutes, seconds)
+                                                    } else {
+                                                        timeRemainingStr = "00:00"
+                                                    }
+                                                } else if (currentState.progress == 1f) {
+                                                    timeRemainingStr = "00:00"
                                                 }
-                                                currentState.copy(systemStats = stats)
+
+                                                currentState.copy(
+                                                    systemStats = stats,
+                                                    estimatedTimeRemaining = timeRemainingStr
+                                                )
                                         }
 
                                         delay(1000)
@@ -191,6 +224,9 @@ class BenchmarkViewModel(
                         }
         }
 
+        // Job for benchmark execution
+        private var benchmarkJob: Job? = null
+
         // NEW IMPLEMENTATION: Delegate to KotlinBenchmarkManager for Single Source of Truth
         fun runBenchmarks(preset: String = "Auto") {
                 // FIX: Reset state immediately to prevent stale navigation
@@ -227,6 +263,12 @@ class BenchmarkViewModel(
                                 // Start performance monitoring for metrics collection
                                 performanceMonitor.start()
 
+                                // Initial Warm-up Phase
+                                _isWarmingUp.value = true
+                                _uiState.value = _uiState.value.copy(currentTestName = "WARMING UP...")
+                                delay(2000) // Simulate warm-up
+                                // Do NOT turn off warm-up here. Wait for first test to start.
+
                                 // 1. RESET STATE - Initialize test states
                                 val testNames =
                                         listOf(
@@ -282,8 +324,34 @@ class BenchmarkViewModel(
                                         // Listen to benchmark events for UI updates
                                         try {
                                                 benchmarkManager.benchmarkEvents.collect { event ->
+                                                        // Handle TEST mode separately (warm-up workload)
+                                                        if (event.mode == "TEST") {
+                                                                when (event.state) {
+                                                                        "STARTED" -> {
+                                                                                _uiState.update { state ->
+                                                                                        state.copy(
+                                                                                                currentTestName = "Warming up device..."
+                                                                                        )
+                                                                                }
+                                                                        }
+                                                                        "COMPLETED" -> {
+                                                                                _uiState.update { state ->
+                                                                                        state.copy(
+                                                                                                currentTestName = "Warm-up complete, starting benchmarks..."
+                                                                                        )
+                                                                                }
+                                                                        }
+                                                                }
+                                                                return@collect // Skip normal processing for TEST events
+                                                        }
+
+                                                        // Normal benchmark event handling (SINGLE and MULTI modes)
                                                         when (event.state) {
                                                                 "STARTED" -> {
+                                                                        // Tests have started, end warm-up state
+                                                                        if (_isWarmingUp.value) {
+                                                                            _isWarmingUp.value = false
+                                                                        }
                                                                         _uiState.update { state ->
                                                                                 state.copy(
                                                                                         currentTestName =
@@ -330,7 +398,8 @@ class BenchmarkViewModel(
                                                                                                                                         "%.2f s",
                                                                                                                                         event.timeMs /
                                                                                                                                                 1000.0
-                                                                                                                                )
+                                                                                                                                ),
+                                                                                                                        durationMs = event.timeMs // Capture duration
                                                                                                                 )
                                                                                                         else
                                                                                                                 it
@@ -391,7 +460,7 @@ class BenchmarkViewModel(
                                 }
 
                                 // Start the benchmark execution
-                                val benchmarkJob =
+                                benchmarkJob =
                                         launch(Dispatchers.IO) {
                                                 benchmarkManager.runAllBenchmarks(preset)
                                         }
@@ -444,12 +513,13 @@ class BenchmarkViewModel(
                                         "Captured performance metrics: ${lastPerformanceMetricsJson.take(100)}..."
                                 )
 
-                                // DIRECT CALLBACK: Call the completion callback directly to ensure
-                                // navigation works
-                                if (onBenchmarkCompleteCallback != null) {
+                                // DIRECT CALLBACK: Emit event regardless of callback presence
+                                // if (onBenchmarkCompleteCallback != null) { // REMOVED
                                         try {
                                                 // Create summary JSON similar to what
                                                 // BenchmarkScreen expects
+                                                val gson = com.google.gson.Gson()
+                                                
                                                 fun sanitize(value: Double): Double =
                                                         when {
                                                                 value.isInfinite() -> 0.0
@@ -457,20 +527,32 @@ class BenchmarkViewModel(
                                                                 else -> value
                                                         }
 
-                                                // Parse performance metrics JSON to avoid
-                                                // double-escaping by Gson
-                                                val performanceMetricsObject =
+                                                // Parse performance metrics JSON into a Map for clean
+                                                // Gson serialization
+                                                val performanceMetricsMap: Map<String, Any> =
                                                         try {
                                                                 if (lastPerformanceMetricsJson
                                                                                 .isNotBlank() &&
                                                                                 lastPerformanceMetricsJson !=
                                                                                         "{}"
                                                                 ) {
-                                                                        JSONObject(
-                                                                                lastPerformanceMetricsJson
+                                                                        val type =
+                                                                                object :
+                                                                                        com.google.gson
+                                                                                        .reflect
+                                                                                        .TypeToken<
+                                                                                                Map<
+                                                                                                        String,
+                                                                                                        Any
+                                                                                                >
+                                                                                        >() {}
+                                                                                        .type
+                                                                        gson.fromJson(
+                                                                                lastPerformanceMetricsJson,
+                                                                                type
                                                                         )
                                                                 } else {
-                                                                        JSONObject()
+                                                                        emptyMap()
                                                                 }
                                                         } catch (e: Exception) {
                                                                 Log.e(
@@ -478,7 +560,7 @@ class BenchmarkViewModel(
                                                                         "Error parsing performance metrics JSON",
                                                                         e
                                                                 )
-                                                                JSONObject()
+                                                                emptyMap()
                                                         }
 
                                                 val summaryData =
@@ -509,7 +591,7 @@ class BenchmarkViewModel(
                                                                 "timestamp" to
                                                                         System.currentTimeMillis(), // Capture completion time
                                                                 "performance_metrics" to
-                                                                        performanceMetricsObject, // FIXED: Use JSONObject instead of String
+                                                                        performanceMetricsMap, // FIXED: Use Map for correct JSON structure
                                                                 "detailed_results" to
                                                                         finalResults.detailedResults
                                                                                 .map { result ->
@@ -532,7 +614,6 @@ class BenchmarkViewModel(
                                                                                 }
                                                         )
 
-                                                val gson = com.google.gson.Gson()
                                                 val summaryJson = gson.toJson(summaryData)
 
                                                 Log.d(
@@ -541,26 +622,24 @@ class BenchmarkViewModel(
                                                 )
                                                 Log.d(
                                                         "BenchmarkViewModel",
-                                                        "Calling onBenchmarkCompleteCallback with JSON (includes performance_metrics): ${summaryJson.take(300)}..."
+                                                        "Emitting completion event with JSON: ${summaryJson.take(300)}..."
                                                 )
-                                                onBenchmarkCompleteCallback!!(summaryJson)
+                                                
+                                                // Emit event for UI to handle navigation
+                                                // Using SharedFlow ensures the UI (even if recreated) can receive it if it's listening
+                                                _completionEvent.emit(summaryJson)
+                                                
                                                 Log.d(
                                                         "BenchmarkViewModel",
-                                                        "onBenchmarkCompleteCallback called successfully"
+                                                        "Completion event emitted successfully"
                                                 )
                                         } catch (e: Exception) {
                                                 Log.e(
                                                         "BenchmarkViewModel",
-                                                        "Error calling onBenchmarkCompleteCallback: ${e.message}",
+                                                        "Error emitting completion event: ${e.message}",
                                                         e
                                                 )
                                         }
-                                } else {
-                                        Log.w(
-                                                "BenchmarkViewModel",
-                                                "onBenchmarkCompleteCallback is null - navigation will rely on flow collection"
-                                        )
-                                }
 
                                 // Save to database with performance metrics
                                 if (historyRepository != null) {
@@ -754,6 +833,21 @@ class BenchmarkViewModel(
                 runBenchmarks(preset)
         }
 
+        fun stopBenchmark() {
+                // Cancel the actual execution job
+                benchmarkJob?.cancel()
+                
+                // Stop the foreground service
+                BenchmarkForegroundService.stop(application)
+                
+                // Reset state
+                isBenchmarkRunning = false
+                _uiState.update { 
+                    it.copy(isRunning = false) 
+                }
+                _benchmarkState.value = BenchmarkState.Idle
+        }
+
         fun saveCpuBenchmarkResult(results: BenchmarkResults, performanceMetricsJson: String = "") {
                 if (historyRepository == null) {
                         Log.w(
