@@ -55,8 +55,9 @@ class GpuFrequencyReader {
     suspend fun readGpuFrequency(): GpuFrequencyState = withContext(Dispatchers.IO) {
         try {
             // Step 1: Check if root is available
-            if (!rootExecutor.hasRootAccess()) {
-                Log.w(TAG, "Root access not available, trying fallback methods")
+            // Use the central RootAccessManager to ensure consistency with the rest of the app
+            if (!RootAccessManager.isRootGranted()) {
+                Log.w(TAG, "Root access not available (checked via RootAccessManager), trying fallback methods")
                 
                 // Try non-root fallback methods
                 val fallbackReader = GpuFrequencyFallback()
